#include <pluginlib/class_list_macros.h>
#include <ros/ros.h>

#include "environment/Environment.h"
#include "exceptions/Exceptions.h"

#include "car/camDataMsg.h"
#include "car/environmentDataMsg.h"
#include "car/ussDataMsg.h"

PLUGINLIB_EXPORT_CLASS(car::Environment, nodelet::Nodelet);

namespace car
{
Environment::Environment(ros::NodeHandle &nh, std::string &name)
    : nh_(nh)
    , name_(name)
    , dt()
    , kalmanFilter(2, buildDynamicModelMatrix(&dt), &dt, buildPath())
    {}
  
Environment::Environment() 
    : dt()
    , kalmanFilter(2, buildDynamicModelMatrix(&dt), &dt, buildPath())
    {}
  
Environment::~Environment() {}
 
void Environment::onInit()
{
    NODELET_INFO("Environment::onInit -- START");
    environmentData = nh_.advertise<environmentDataMsg>("environmentData", 1);
    ussData = nh_.subscribe("ussData", 1, &Environment::ussDataCallback, this);
    camData = nh_.subscribe("camData", 1, &Environment::camDataCallback, this);
    NODELET_INFO("Environment::onInit -- END");
}

void Environment::ussDataCallback(const ussDataMsg::ConstPtr& inMsg)
{
    // std::cout << "Environment recived new uss data (" << inMsg->distance << ").\n";
    // distance = inMsg->distance;
    // relativeSpeed = 1.0;
    
    // environmentDataMsg outMsg;
    // outMsg.distance = distance;
    // outMsg.relativeSpeed = relativeSpeed;
    
    // environmentData.publish(outMsg);
}

void Environment::camDataCallback(const camDataMsg::ConstPtr& inMsg)
{
    // std::cout << "Environment recived new cam data ( ).\n";
}

std::string Environment::buildPath()
{
    std::string userHome = getenv("HOME");
    return userHome + "/CarConfig/kalman.config";
}

matrix<double> Environment::buildDynamicModelMatrix(double *deltaT)
{
    /* TAKE CARE ON THE DIMENSION */
    *deltaT = 1.0;
    matrix<double> dynamicModel(2,2);
    dynamicModel(0,0) = 1.0,
    dynamicModel(0,1) = 1.0/(*deltaT);
    dynamicModel(1,0) = 0;
    dynamicModel(1,1) = 1.0;
    return dynamicModel;
}
}
