#include "environment/KalmanFilter.h"
#include "exceptions/Exceptions.h"
#include <iterator>

#include <stdio.h>

namespace car
{
KalmanFilter::KalmanFilter(vector<double> x, matrix<double> P, matrix<double> F, matrix<double> Q, vector<double> H, vector<double> R, double *deltaT)
    : x(x)
    , P(P)
    , F(F)
    , Q(Q)
    , H(H)
    , R(R)
    , dt(deltaT)
    {}

KalmanFilter::KalmanFilter(int dimension, matrix<double> F, double *deltaT, std::string configFilePath)
    : x(dimension)
    , P(dimension, dimension)
    , F(F)
    , Q(dimension, dimension)
    , H(dimension)
    , R(dimension)
    , dt(deltaT)
    {   
        readConfigFile(configFilePath);
    }

KalmanFilter::~KalmanFilter() {}

std::vector<std::string> KalmanFilter::getNextFileLine(std::ifstream *file)
{
    // read a line
    std::string contentLine;
    std::getline(*file, contentLine);
        
    // split this line
    std::istringstream iss(contentLine);
    std::vector<std::string> words {std::istream_iterator<std::string>(iss), {} };
    return words;
}

void KalmanFilter::readConfigFile(std::string configFilePath)
{
    // open config file
    std::ifstream configFile;
    configFile.open(configFilePath, std::ifstream::in);
    if (!configFile.is_open()) { throw FileNotFound(); }

    // read file
    while (!configFile.eof())
    {
        std::vector<std::string> words = getNextFileLine(&configFile);
        if ( words.size() == 3 ) // this line contains parameter[at (0)] and value [at (1)]
        {
            if ( words.at(0) == "//") { continue; }
            int rows = std::stoi(words.at(1));
            int cols = std::stoi(words.at(2));
            if (words.at(0) == "x") {
                int i = 0;
                while ( i < rows )
                {
                    words = getNextFileLine(&configFile);
                    x(i++) = std::stod(words.at(0));
                }
            } else if (words.at(0) == "P"){
                int i = 0;
                while ( i < rows )
                {
                    words = getNextFileLine(&configFile);
                    int j = 0;
                    while ( j < cols )
                    {
                        P(i, j) = std::stod(words.at(j));
                        j++;
                    }
                    i++;
                }
            } else if (words.at(0) == "Q"){
                int i = 0;
                while ( i < rows )
                {
                    words = getNextFileLine(&configFile);
                    int j = 0;
                    while ( j < cols )
                    {
                        Q(i, j) = std::stod(words.at(j));
                        j++;
                    }
                    i++;
                }
            } else if (words.at(0) == "H"){
                int i = 0;
                while ( i < rows )
                {
                    words = getNextFileLine(&configFile);
                    H(i++) = std::stod(words.at(0));
                }
            } else if (words.at(0) == "R"){
                int i = 0;
                while ( i < rows )
                {
                    words = getNextFileLine(&configFile);
                    x(i++) = std::stod(words.at(0));
                }
            }       
        }
    }
    configFile.close();
    return;
}

void KalmanFilter::predict(){
    // x = F*x
    // P = (F * P * F.T) + Q
}

void KalmanFilter::update(vector<double> mesVec){
    // Innovation
    // y = mesVec.T, - H * x

    // Residualkovarianz
    // S = H * P * H.T + R

    // Kalman - Gain
    // K = P * H.T * S**(-1)
    
    // update the predecited values
    // x = x + K * y
    // P = P - K * S * K.T
}

vector<double> KalmanFilter::nextKalmanIteration(vector <double> mesVec)
{
    update(mesVec);
    predict();
    return x;
}
}
